//	Copyright (c) 1988-93 by Abraxas Software.

//  This file can be used by company programmers to
//  monitor their compliance with commonly-accepted
//  standards for C source code maintainability.

//  The warnings are intended to be seen in the context of
//  a list file. Use:  check -Rmaintain -L myproject.ccp

//  Warning codes:  1000  lexical
//                  2000  preprocessor
//                  3000  declaration
//                  4000  expression
//                  5000  function
//                  6000  comment
//                  7000  general

//  These rules may serve as a basis for customizing your
//  own set of corporate standards for C source code.

#include <check.cch>

int		k;

/********************* LEXICAL RULES *********************/

if ( lex_not_manifest )
	if ( ! lex_initializer )
		warn( 1000, "Magic number %s: use a macro constant here.", token() );

if ( lex_big_octal )
	warn( 1001, "Digits 8 and 9 are not octal." );

if ( lex_bad_call )
	warn( 1002, "Wrong number of arguments in this macro call." );

if ( lex_char_empty )
	warn( 1003, "Do not use empty character constants." );

if ( macro("offsetof") )
	warn( 1004, "Do not use offsets!" );

if ( lex_num_escape == 7 )
	warn( 1005, "If you need an audible alarm, use \\a instead of \\07." );

if ( lex_hex_escape > 2 )
	warn( 1006, "There are %d digits in this hex escape sequence.", lex_hex_escape );


/********************* PREPROCESSOR RULES *********************/

if ( pp_lowercase)
	warn( 2000, "Always spell macro names in uppercase." );

if ( dcl_all_upper )
	warn( 2001, "ALL UPPERCASE identifiers should be reserved for macro names." );

if ( pp_arg_count > 4 )
	warn( 2002, "Too many parameters in this macro." );

if ( pp_recursive )
	warn( 2003, "Danger: recursive macro definition." );

if ( pp_semicolon )
	warn( 2004, "Macro body ends with semicolon - is this OK??" );

if ( pp_arg_multiple )
	warn( 2005, "Possible undesired side-effects may occur here." );

if ( pp_arg_paren )
	warn( 2006, "Macro argument not enclosed in parentheses." );

if ( pp_assign )
	warn( 2007, "The \"=\" sign in this macro may be an error." );

if ( pp_not_defined )
	warn( 2008, "This expression depends on an undefined macro." );

if ( pp_depend )
	warn( 2009, "Another macro depends on this macro - do not #undef." );

if ( pp_sub_keyword )
	warn( 2010, "Do not redefine keywords." );

if ( pp_stack )
	warn( 2011, "This macro hides another!" );

if ( pp_unstack )
	warn( 2012, "This #undef exposes a hidden macro definition." );

if ( pp_comment )
	warn( 2013, "Separate comments from tokens with whitespace, or use the paste operator." );

if ( pp_macro_conflict )
	warn( 2014, "This macro was defined differently in file %s, line %d.",
			conflict_file(), conflict_line );


/********************* DECLARATION RULES *********************/

if ( dcl_all_upper )
	if ( dcl_ident_length > 1 )
		warn( 3000, "Only macro names should be all uppercase." );

if ( dcl_ident_length < 3 )
	if ( ! dcl_extern )
		warn( 3001, "Use 3 or more characters for identifiers." );

if ( dcl_variable )
	{
	if ( dcl_first_upper && dcl_definition )
		warn( 3002, "Use a lowercase initial letter for variable %s.", dcl_name() );
	if ( lin_dcl_count > 1 )
		warn( 3003, "Put each declarator variable on a separate line." );
	}

if ( dcl_function  )
	{
	if ( (! dcl_first_upper) && dcl_definition )
		warn( 3004, "Use an uppercase initial letter for function %s.", dcl_name() );
	if ( dcl_count > 1 )
		warn( 3005, "Declare only one function in this statement." );
	}

if ( dcl_hidden )
	warn( 3006, "Declarator %s hides another - use another name.", dcl_name() );

if ( dcl_enum_hidden )
	warn( 3006, "Declarator %s hides an enumerated constant.", dcl_name() );

if ( dcl_typedef )
	{
	if ( dcl_function )
		warn( 3007, "Function types should not be typedef\'ed." );
	if ( ! dcl_first_upper )
		warn( 3008, "Use an uppercase initial letter for typedef %s.", dcl_name() );
	}

if ( dcl_local )
	{
	if ( dcl_tag_def )
		warn( 3009, "NEVER introduce new struct or union tags at the local level." );
	if ( dcl_extern )
		warn( 3010, "External variables with local scope should be avoided." );
	if ( dcl_typedef )
		warn( 3011, "Typedef names with local scope should be avoided." );
	}

if ( dcl_empty )
	if ( ! dcl_tag_def )
		warn( 3012, "Empty declaration." );

if ( dcl_no_specifier )
	{
	if ( dcl_function )
		warn( 3013, "A return type must be specified for %s().", dcl_name() );
	else
		warn( 3014, "An explicit type must be specified for %s.", dcl_name() );
	}

if ( dcl_extern )
	if ( dcl_initializer )
		warn( 3015, "NEVER initialize an explicitly extern variable." );

if ( dcl_bitfield_anon )
	warn( 3016, "All bitfields must have names." );

if ( dcl_union_init )
	warn( 3017, "Do not initialize unions." );

if ( dcl_auto_init )
	warn( 3018, "Do not initialize auto structs or arrays." );

if ( dcl_union_bits )
	warn( 3019, "Do not use bitfields in unions." );

if ( dcl_tag_def )
	if ( lin_source && (dcl_base == STRUCT_TYPE) )
		warn( 3020, "Define structures in header files only." );

if ( dcl_parameter )
	if ( dcl_abstract )
		warn( 3021, "A parameter name would help here." );

if ( dcl_conflict )
	warn( 3022, "%s was declared differently in file %s, line %d.",
			dcl_name(), conflict_file(), conflict_line );

if ( tag_hidden )
	warn( 3023, "Tag %s hides another - use another name.", tag_name() );


/********************* EXPRESSION RULES *********************/

if ( idn_no_prototype )
	warn( 4001, "Function %s needs a prototype.", idn_name() );

if ( stm_cp_assign > 1 )
	warn( 4002, "Use no more than one compound assignment per statement." );

if ( op_open_funargs )
	{
	if ( strcmp(op_function(),"gets") == 0 )
		warn( 4003, "Use fgets() instead." );
	}


/********************* FUNCTION RULES *********************/

int		nReturns;			//	Number of returns per function.

if ( fcn_begin )
	nReturns = 0;			//  Initialize the "return" counter in every function.

if ( keyword("return") )
	{
	++nReturns;				//  Flag functions with more than one return.
	if ( nReturns > 1 )
		warn( 5000, "Use only one return per function." );
	}

if ( fcn_end )				//  Flag functions with no return.
	if ( nReturns == 0 )
		warn( 5001, "This function needs an explicit return." );

if ( dcl_oldstyle )
	warn( 5002, "Function %s must be defined in prototype form.", dcl_name() );

if ( dcl_parm_hidden )
	warn( 5003, "Local identifier %s hides a parameter of function %s.", dcl_name(), fcn_name() );

if ( fcn_end )
	if ( fcn_exec_lines > 250 )
		warn( 5004, "Function %s has too many lines.", fcn_name() );

if ( stm_return_void )
	warn( 5005, "Returned value conflicts with function type." );


/********************* COMMENT RULES *********************/

if ( lin_dcl_count == 1 )
	if ( ! lin_has_comment )
		warn( 6000, "Descriptive comment missing." );


/********************* GENERAL RULES *********************/

if ( test_needed("malloc") )
	warn( 7000, "malloc return value was not tested!" );

if ( test_needed("calloc") )
	warn( 7000, "calloc return value was not tested!" );

if ( test_needed("alloc") )
	warn( 7000, "alloc return value was not tested!" );

if ( op_high_prefix )
	if ( op_white_after )
		warn( 7001, "Remove the space before this operator." );

if ( op_high_postfix )
	if ( op_white_before )
		warn( 7002, "Remove the space before this operator." );

if ( op_high_infix )
	{
	if ( op_white_before )
		warn( 7003, "Remove the space before this operator." );
	if ( op_white_after )
		warn( 7003, "Remove the space after this operator." );
	}

if ( op_low )
	{
	if ( ! op_white_before )
		warn( 7004, "Put space before this operator." );
	if ( ! op_white_after )
		warn( 7004, "Put space after this operator." );
	}

if ( stm_goto )
	warn( 7005, "Initialization may fail on goto from outside this block." );

if ( stm_no_break )
	warn( 7006, "Previous case has no break - is this OK??" );

if ( stm_unused )
	{
	k = 0;
	while ( k < stm_unused )
		warn( 7007, "Variable %s was declared but not used.", stm_unused_name(k++) );
	}

if ( stm_semicolon )
	warn( 7008, "This semicolon may be an error." );

if ( stm_no_default )
	warn( 7009, "This switch has no default case." );

if ( stm_loop_back )
	warn( 7010, "Do not use backward goto statements." );

if ( idn_no_init )
	warn( 7011, "Local variable %s used before initialized.", idn_name() );
